clear all;
% inport simulation data
load Data
ID=squeeze(data(1,:,:)); 
ag=squeeze(data(2,:,:));
tsi=squeeze(data(3,:,:));
% ID is a NxM matrix, covering a period of M days. ID contains the id-number of
% every individual present in the population at each day, tsi and ag correspond to 
% the time since infection and the age, respectively.

% the parameters in this program are chosen to reproduce the big dot in
% Fig.5(d), the curves in Fig.6(d) and the scatter plot in Fig.7(d). 

% other results can be obtained by changing the parameters, and by a
% different choice of the a priori distribution, the information used
% (see theta_a,theta_b,theta_e,theta_aa). also, the presence-absence
% situation can be modeled by a different choice of Ptl, see below. 


dt=28; %time between trappings (in days)
ptrap=0.35;%trapping probability

%%
load Ltemplate Ltemplate  %Ltemplate is the Ab-level function
Level=[zeros(5000,1); max(Ltemplate([1:5000]-1),1)];
Level(170+5000:end)=7.3;
STD=1;

excRNA=[zeros(5000,1); pEXC([1:5000]'-1)];
bloodRNA=[zeros(5000,1); pBLOOD([1:5000]'-1)];

figure; plot(Level(5000:5100));
figure; plot(excRNA(5000:5100));
figure; plot(bloodRNA(5000:5100));

Trange=460; % 2*Trange defines the range of possible time of infections, centred around the first trapping, see below.

id=setdiff(unique(ID),-666);  % -666 mark empty cells in population matrix
[n,nt]=size(tsi);
Time=[-Trange:nt];

aantalI=(1:nt)*0; % total number of infected animals trapped
aantalS=(1:nt)*0; % total number of suceptible animals trapped

% calculation proportion trapped
lambda=0.008537; % 1/lambda calculated to be the average lifespan of an animal in our simulation
x=[1:10000];
offset=sum(exp(-lambda*x)*lambda.*(1-(1-ptrap).^(x/dt)));

% initialize everything to initial value of -999
nid=length(id); %total number of individuals
toi=zeros(nid,1)-999;
theta_a=zeros(nid,1)-999; theta_b=zeros(nid,1)-999; theta_e=zeros(nid,1)-999; theta_aa=zeros(nid,1)-999;
entropy_a=zeros(nid,1)-999; entropy_b=zeros(nid,1)-999; entropy_e=zeros(nid,1)-999; entropy_aa=zeros(nid,1)-999;
N=zeros(nid,1)-999;

% loop over all individuals in the populations
for k=1:length(id)
    k
% select the times when the individual is in the population
    IDX=find(ID==id(k));
    idx=(find(sum(ID==id(k))==1));
    BD(k)=idx(1)-ag(IDX(1)); %calculate day of birth

    par=idx(tsi(IDX)>0);
    if isfinite(par)
        toi(k)=par(1)-max(tsi(IDX(1)),1); %time of infection of the animal
    end
% select the times when the individual is trapped
    idxpar=find((rem(idx,dt)==0)&((rand(1,length(idx))<ptrap)));
    idx=idx(idxpar);
    IDX=IDX(idxpar);
    par=idx(tsi(IDX)>0);
    idxS=setdiff(idx,par);
    idxI=par; 
    
    if (1-isempty(idx))&&(tsi(IDX(end))>0)
    N(k)=length(idx); 
        
% generate ab levels for every caught individual 
    Cov=eye(length(idx))*STD^2;
    Tpar=max(min(tsi(IDX),170),1);
    li=max(mvnrnd(Ltemplate(Tpar),Cov),1);    
    li(tsi(IDX)==0)=0;
    exci=rand(length(IDX),1)<pEXC(tsi(IDX));
    bloodi=rand(length(IDX),1)<pBLOOD(tsi(IDX));

% calculate the covariance matrix
    par=(1:length(idx))*STD^2;
    par(li==0)=STD^2;
    Cov=diag(par);
    invC=inv(Cov);
% calculate the likelihood for the different toi's
    idx2=idx-idx(1); 
    for ti=1:2*Trange+1 %ti is time since enfection with respect to the time of the first trapping
        %ti = Trange+1 infection occurs at first trapping idx(1); 
        %ti < Trange+1 infection occurs later;
        %ti > Trange+1 infection occurs earlier 
        C=(li'-Level(5001-Trange+ti+idx2))'*invC*(li'-Level(5001-Trange+ti+idx2));
% in case of Ab-level information
        Ptl(ti)=exp(-0.5*C)';
% in case of Ab presence/absence information        
%        Ptl(ti)=all(li==Level(5001-Trange+ti+idx2));
        Pexc(ti)=prod(exci.*(2*excRNA(5001-Trange+ti+idx2)-1)+(1-excRNA(5001-Trange+ti+idx2)));
        Pblood(ti)=prod(bloodi.*(2*bloodRNA(5001-Trange+ti+idx2)-1)+(1-bloodRNA(5001-Trange+ti+idx2)));
    end
    
% model prior distribtuion
    Psurvival=ones(1,2*Trange+1);
    Psurvival(1:Trange+1)=exp(-(Trange:-1:0)*lambda);        %if survival is based on mortality rate
    %Psurvival(1:Trange+1)=(1-ones(1,Trange+1)*ptrap).^((Trange:-1:0)/dt); %if survival is based on trapping probability
    Psurvival=fliplr(Psurvival);
    
    %no age information, only include that the maximal age is 450 days
    Pboundary=zeros(1,2*Trange+1);
    Pboundary(Trange+1+idx2(end)-450:Trange+1+idx2(end))=1;
    Pboundary=fliplr(Pboundary);
    
    %if the exact of the animal is known
    Page=zeros(1,2*Trange+1);
    Page(Trange+1-ag(IDX(1)):Trange+1-ag(IDX(1))+450)=1;
    Page=fliplr(Page);

%%% only ab-levels
    Pt=Ptl.*Pboundary.*Psurvival;
    Pt=Pt/sum(Pt);
    entropy_a(k)=-sum(Pt.*log2(Pt+eps));
    I=min(max(round(Pt*(1:length(Pt))'),1),length(Pt));        
    theta_a(k)=Trange+1-I+min(idx);

%%% ab-levels + virus in blood   
    Pt=Ptl.*Pboundary.*Pblood.*Psurvival; 
    Pt=Pt/sum(Pt);
    entropy_b(k)=-sum(Pt.*log2(Pt+eps));
    I=min(max(round(Pt*(1:length(Pt))'),1),length(Pt));        
    theta_b(k)=Trange+1-I+min(idx);
    
%%% ab-levels + virus in excretions   
    Pt=Ptl.*Pboundary.*Pblood.*Pexc.*Psurvival; 
    Pt=Pt/sum(Pt+eps);
    entropy_e(k)=-sum(Pt.*log2(Pt+eps));
    I=min(max(round(Pt*(1:length(Pt))'),1),length(Pt));        
    theta_e(k)=Trange+1-I+min(idx);
    
%%% ab-levels + virus in excretions + virus in blood + exact knowledge of age    
    Pt=Ptl.*Pboundary.*Page.*Pblood.*Pexc;
    Pt=Pt/sum(Pt);
    entropy_aa(k)=-sum(Pt.*log2(Pt+eps));
    I=min(max(round(Pt*(1:length(Pt))'),1),length(Pt));        
    theta_aa(k)=Trange+1-I+min(idx);
    end
    aantalI(idxI)=aantalI(idxI)+1; %number of infected animals trapped
    aantalS(idxS)=aantalS(idxS)+1; %number of susceptible animals trapped
end

entropy=entropy_aa;
theta=theta_aa;
A=hist(toi(toi>0),Time+0.5);      
Et=100; %define threshold on the entropy  
Ttot=hist(theta((entropy<Et)&(entropy>0)),Time+0.5);
offset2=offset*length(entropy((entropy<Et)&(entropy>0)))/(length(entropy(entropy>0)));
MSE=sqrt(mean((toi((entropy<Et)&(entropy>0))-theta((entropy<Et)&(entropy>0))).^2));

figure; plot(Time(1:end-1),A(2:end)); hold on; plot(Time,Ttot/offset,'g');
n=5;    
T=[0:length(Ttot)+(n-1)-1];
figure; plot(T,conv(A,ones(1,n)/n)); hold on; plot(T,conv(Ttot/offset,ones(1,n)/n),'g');
figure; scatter(toi(N>0),toi(N>0)-theta(N>0),20,entropy(N>0),'o','filled');



